package org.zheng.demo;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.time.LocalDate;

import org.junit.jupiter.api.Test;
import org.zheng.demo.data.Person;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

class TestJavaTimeModule {

	@Test
	void test_enable_objectMapper_jsr310() {
		ObjectMapper ob = new ObjectMapper();
		ob.registerModule(new JavaTimeModule());

		Person per = new Person("Zheng");
		per.setLocalDateJsr310(LocalDate.of(2020, 01, 01));
		per.setLocalDateObj(LocalDate.of(1970, 01, 01));

		try {
			String jsonString = ob.writeValueAsString(per);
			assertEquals(
					"{\"age\":0,\"name\":\"Zheng\",\"localDateObj\":[1970,1,1],\"localDateJsr310\":\"Wed 01-Jan-2020 AD\"}",
					jsonString);
			System.out.println(ob.writerWithDefaultPrettyPrinter().writeValueAsString(per));

			Person readPer = ob.readValue(jsonString, Person.class);
			assertTrue(per.equals(readPer));
			assertEquals("Zheng", readPer.getName());
			assertEquals(0, readPer.getAge());
			assertEquals(LocalDate.of(1970, 1, 1), readPer.getLocalDateObj());
			assertEquals(LocalDate.of(2020, 1, 1), readPer.getLocalDateJsr310());

		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}

	}

}
