package org.zheng.demo;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.time.LocalDate;

import org.junit.jupiter.api.Test;
import org.zheng.demo.data.Person;
import org.zheng.demo.module.CustLocalDateDeserializer;
import org.zheng.demo.module.CustLocalDateSerializer;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.exc.InvalidDefinitionException;
import com.fasterxml.jackson.databind.module.SimpleModule;

public class TestSimpleModule {
	ObjectMapper ob = new ObjectMapper();
	Person per = new Person("Zheng");
	SimpleModule module = new SimpleModule();

	@Test
	void test_simpleModule_with_cust_serializer() throws JsonProcessingException {
		// Registering the customized serialize and de-serialize
		module.addSerializer(LocalDate.class, new CustLocalDateSerializer());
		module.addDeserializer(LocalDate.class, new CustLocalDateDeserializer());

		// Register the module with the ObjectMapper
		ob.registerModule(module);

		per.setLocalDateJsr310(LocalDate.of(2020, 01, 01));
		per.setLocalDateObj(LocalDate.of(1970, 01, 01));

		String jsonString = ob.writeValueAsString(per);
		System.out.println(ob.writerWithDefaultPrettyPrinter().writeValueAsString(per));

		Person readPer = ob.readValue(jsonString, Person.class);
		assertTrue(per.equals(readPer));
		assertEquals("Zheng", readPer.getName());
		assertEquals(0, readPer.getAge());
		assertEquals(LocalDate.of(1970, 1, 1), readPer.getLocalDateObj());
		assertEquals(LocalDate.of(2020, 1, 1), readPer.getLocalDateJsr310());
	}

	@Test
	void test_simpleModule_with_serialize_then_deserialze_got_exception() throws JsonProcessingException {
		// Registering the customized serialize only
		module.addSerializer(LocalDate.class, new CustLocalDateSerializer());

		// Register the module with the ObjectMapper
		ob.registerModule(module);

		per.setLocalDateJsr310(LocalDate.of(2020, 01, 01));
		per.setLocalDateObj(LocalDate.of(1970, 01, 01));
		String jsonStr = ob.writeValueAsString(per);
		System.out.println(ob.writerWithDefaultPrettyPrinter().writeValueAsString(per));

		InvalidDefinitionException exception = assertThrows(InvalidDefinitionException.class,
				() -> ob.readValue(jsonStr, Person.class));
		assertTrue(exception.getMessage().contains("Java 8 date/time type `java.time.LocalDate`"));
	}

	@Test
	void test_simpleModule_ok_at_null() throws JsonProcessingException {
		// Registering the customized serialize only
		module.addSerializer(LocalDate.class, new CustLocalDateSerializer());

		// Register the module with the ObjectMapper
		ob.registerModule(module);

		per.setLocalDateJsr310(LocalDate.of(2020, 01, 01));

		String jsonStr = ob.writeValueAsString(per);
		System.out.println(ob.writerWithDefaultPrettyPrinter().writeValueAsString(per));

		Person readPer = ob.readValue(jsonStr, Person.class);
		assertEquals("Zheng", readPer.getName());
		assertEquals(0, readPer.getAge());
		assertEquals(LocalDate.of(2020, 1, 1), readPer.getLocalDateJsr310());
	}
}
